@@ -2,11 +2,15 @@
 #
 #  libzvbi test
 #  Copyright (C) 2006 Michael H. Schimek
-#  Perl Port: Copyright (C) 2007 Tom Zoerner
+#  Perl Port: Copyright (C) 2007, 2020 Tom Zoerner
 #
 
-# Perl $Id: test-vps.pl,v 1.1 2007/11/18 18:48:35 tom Exp tom $
-# #Id: test-vps.c,v 1.3 2006/10/08 06:19:48 mschimek Exp $
+# Description:
+#
+#   This script contains tests for encoding and decoding the VPS data
+#   service on randomly generated data.
+#
+#   (This is a direct translation of test/test-vps.c in libzvbi.)
 
 use strict;
 use blib;
@@ -28,7 +32,7 @@ sub main
 	$buffer2 = pack("C13", @rands);
 	$buffer1 = $buffer2;
 
-        $cni2 = Video::ZVBI::decode_vps_cni($buffer2);
+        $cni2 = Video::ZVBI::decode_vps_cni ($buffer2);
 
         $buffer1 = Video::ZVBI::encode_vps_cni ($cni2);
         die unless defined $buffer1;
@@ -50,6 +54,5 @@ sub main
 	$buffer1 = Video::ZVBI::encode_vps_cni ((1<<31)-1) and die;
 }
 
-srand;
+srand();
 main();
-
